package Poet::Mason::Plugin;
BEGIN {
  $Poet::Mason::Plugin::VERSION = '0.03';
}
use Moose;
with 'Mason::Plugin';

1;

__END__
=pod

=head1 SEE ALSO

L<Poet|Poet>

=head1 AUTHOR

Jonathan Swartz <swartz@pobox.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2012 by Jonathan Swartz.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

