package Poet::Plack::Response;
BEGIN {
  $Poet::Plack::Response::VERSION = '0.03';
}
use Poet::Moose;
extends 'Plack::Response';

1;



=pod

=head1 NAME

Poet::Plack::Response - Poet's subclass of Plack::Response

=head1 DESCRIPTION

This is a Poet-specific subclass of L<Plack::Response|Plack::Response>,
reserved for future additions and overrides.

=head1 SEE ALSO

L<Poet|Poet>

=head1 AUTHOR

Jonathan Swartz <swartz@pobox.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2012 by Jonathan Swartz.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut


__END__

